@@ -4,6 +4,7 @@
 # These are various helpers for "special" attributes
 
 import copy
+import weakref
 
 from collections import OrderedDict
 
@@ -20,7 +21,41 @@ class SpecialAttributesDict(dict):
     # A dict that supports returning special members
     def __init__(self, tag):
         dict.__init__(self)
-        self.tag = tag
+
+        self._setTag(tag)
+
+
+    @property
+    def tag(self):
+        '''
+            tag - Property (dot-access) for the associated tag to this attributes dict
+
+                  Handles getting the value from a weak association
+
+                    @return <AdvancedTag/None> - The associated tag, or None if no association
+        '''
+
+        tagRef = self._tagRef
+        if tagRef:
+            return tagRef()
+        else:
+            return None
+
+    def _setTag(self, tag):
+        '''
+            _setTag - INTERNAL METHOD. Associated a given AdvancedTag to this attributes dict.
+
+                        If bool(#tag) is True, will set the weakref to that tag.
+
+                        Otherwise, will clear the reference
+
+                      @param tag <AdvancedTag/None> - Either the AdvancedTag to associate, or None to clear current association
+        '''
+        if tag:
+            self._tagRef = weakref.ref(tag)
+        else:
+            self._tagRef = None
+
 
     def __getitem__(self, key):
         if key == 'style':
@@ -33,25 +68,59 @@ class SpecialAttributesDict(dict):
         except KeyError:
             return None #  TODO: support undefined?
 
+
+    # TODO: Implement __delitem__
+
     def get(self, key, default=None):
+        '''
+            get - Gets an attribute by key with the chance to provide a default value
+
+                @param key <str> - The key to query
+
+                @param default <Anything> Default None - The value to return if key is not found
+
+             @return - The value of attribute at #key, or #default if not present.
+        '''
+
         if key in {'style', 'class'} or key in self.keys():
             return self[key]
         return default
 
     def _direct_set(self, key, value):
+        '''
+            _direct_set - INTERNAL USE ONLY!!!!
+
+                Directly sets a value on the underlying dict, without running through the setitem logic
+
+        '''
         dict.__setitem__(self, key, value)
         return value
 
+    def _direct_del(self, key):
+        '''
+            _direct_set - INTERNAL USE ONLY!!!!
+
+                Directly deletes a value from the underlying dict, without running through logic
+
+        '''
+        try:
+            dict.__delitem__(self, key)
+        except:
+            pass
+
     def __setitem__(self, key, value):
+        
+        tag = self.tag
+
         if key == 'style':
             if not isinstance(value, StyleAttribute):
-                self.tag.style = StyleAttribute(value, self.tag)
+                tag.style = StyleAttribute(value, tag)
             else:
-                self.tag.style = value
+                tag.style = value
         elif key == 'class':
 
             # Ensure when we update the "class" attribute, that we update the list as well.
-            self.tag.classNames = [x for x in value.split(' ') if x]
+            tag.classNames = [x for x in value.split(' ') if x]
             dict.__setitem__(self, 'class',  value)
             return value
 
@@ -223,7 +292,7 @@ class StyleAttribute(object):
         StyleAttribute - Represents the "style" field on a tag.
     '''
 
-    RESERVED_ATTRIBUTES = ('_styleValue', '_styleDict', '_asStr', 'tag')
+    RESERVED_ATTRIBUTES = ('_styleValue', '_styleDict', '_asStr', '_ensureHtmlAttribute', 'tag', '_tagRef', 'setTag', 'isEmpty', 'setProperty')
 
     def __init__(self, styleValue, tag=None):
         '''
@@ -236,7 +305,49 @@ class StyleAttribute(object):
 
         self._styleValue = styleValue
         self._styleDict = StyleAttribute.styleToDict(styleValue)
-        self.tag = tag
+
+        self.setTag(tag)
+
+        # Set the attribute in the tag html if necessary, or clear it.
+        self._ensureHtmlAttribute()
+
+    @property
+    def tag(self):
+        '''
+            tag - Property (dot-access variable) which will return the associated tag, if any.
+            
+              This method should be used for access to handle the weakref.
+
+              @see setTag - Method to set or remove the tag association
+
+                @return <AdvancedTag/None> - If a tag is associated with this style, it will be returned.
+                                                Otherwise, None will be returned
+        '''
+        tagRef = object.__getattribute__(self, '_tagRef')
+        if tagRef:
+            return tagRef()
+
+        return None
+
+    def setTag(self, tag):
+        '''
+            setTag - Set the tag association for this style.
+            
+              This will handle the underlying weakref to the tag.
+
+              Call setTag(None) to clear the association, otherwise setTag(tag) to associate this style to that tag.
+
+
+                @param tag <AdvancedTag/None> - The new association. If None, the association is cleared, otherwise the passed tag
+                    becomes associated with this style.
+
+        '''
+                
+        if tag:
+            self._tagRef = weakref.ref(tag)
+        else:
+            self._tagRef = None
+
 
     def __getattribute__(self, name):
         '''
@@ -248,7 +359,7 @@ class StyleAttribute(object):
 
             @return <str> - The attribute value or empty string if not set
         '''
-        if name in StyleAttribute.RESERVED_ATTRIBUTES:
+        if name in StyleAttribute.RESERVED_ATTRIBUTES or name.startswith('__'):
             return object.__getattribute__(self, name)
 
         dashName = StyleAttribute.camelCaseToDashName(name)
@@ -276,20 +387,99 @@ class StyleAttribute(object):
         
         attrName = StyleAttribute.camelCaseToDashName(name)
 
+        styleDict = self._styleDict
+
         if attrName != name:
             name = attrName
 
         if not val:
-            if name in self._styleDict:
-                del self._styleDict[name]
+            if name in styleDict:
+                del styleDict[name]
         else:
-            self._styleDict[name] = val
-
-        if self.tag and self._styleDict:
-            self.tag.setAttribute('style', self)
+            styleDict[name] = val
 
+        # Okay, since we don't want empty style="" on every tag, we have to attach and remove as properties change
+        self._ensureHtmlAttribute()
+                
         return val
 
+    def _ensureHtmlAttribute(self):
+        '''
+            _ensureHtmlAttribute - INTERNAL METHOD. 
+                                    Ensure the "style" attribute is present in the html attributes when
+                                        is has a value, and absent when it does not.
+
+              This requires special linkage.
+        '''
+        tag = self.tag
+
+        if tag:
+            styleDict = self._styleDict
+            tagAttributes = tag._attributes
+
+            # If this is called before we have _attributes setup
+            if not issubclass(tagAttributes.__class__, SpecialAttributesDict):
+                return
+
+            # If we have any styles set, ensure we have the style="whatever" in the HTML representation,
+            #   otherwise ensure we don't have style="" 
+            if not styleDict:
+                tagAttributes._direct_del('style')
+            else: #if 'style' not in tagAttributes.keys():
+                tagAttributes._direct_set('style', self)
+
+    def isEmpty(self):
+        '''
+            isEmpty - Check if this is an "empty" style (no attributes set)
+
+                @return <bool> - True if no attributes are set, otherwise False
+        '''
+
+        return bool( len(self._styleDict) == 0)
+
+
+    def setProperty(self, name, value):
+        '''
+            setProperty - Set a style property to a value.
+
+                NOTE: To remove a style, use a value of empty string, or None
+
+                 @param name <str> - The style name.
+
+                    NOTE: The dash names are expected here, whereas dot-access expects the camel case names.
+
+                      Example:  name="font-weight"  versus the dot-access  style.fontWeight
+
+                 @param value <str> - The style value, or empty string to remove property
+        '''
+        styleDict = self._styleDict
+
+        if value in ('', None):
+            try:
+                del styleDict[name]
+            except KeyError:
+                pass
+        else:
+            styleDict[name] = str(value)
+                
+
+#            if newValue:
+#                # If replacing, just set/override it
+#                styleDict[newName] = newValue
+#            elif newName in styleDict:
+#                # Delete if present and empty value passed
+#                del styleDict[newName]
+#
+#        if styleDict:
+#            # If anything left, build a str
+#            styleStr = '; '.join([name + ': ' + value for name, value in styleDict.items()])
+#            self.setAttribute('style', styleStr)
+#            return styleStr
+#        else:
+#            # Last item removed, so remove attribute.
+#            self.removeAttribute('style')
+#            return ''
+
     @staticmethod
     def dashNameToCamelCase(dashName):
         '''
@@ -354,82 +544,90 @@ class StyleAttribute(object):
 
         return styleDict
 
-    def getStyleDict(self):
-        '''
-            getStyleDict - Returns a dictionary of style attribute name:value. 
+    # NOTE: THE MAJORITY OF THESE BELOW ARE DISABLED (not in reserved attributes)
 
-            @return dict <str:str> style attribute names to values. Impl is OrderedDict and will match positioning in string value
-        '''
-        return self._styleDict
-
-    def getStyle(self, styleName):
-        '''
-            getStyle - Gets the value of a style paramater, part of the "style" attribute
-
-            @param styleName - The name of the style
-
-              NOTE: dash-names (like padding-top) are expected here
-
-            @return - String of the value of the style. '' is no value.
-        '''
-        return self.getStyleDict().get(styleName.lower()) or ''
-        
-
-    def setStyle(self, styleName, styleValue):
-        '''
-            setStyle - Sets a style param. Example: "display", "block"
-
-                If you need to set many styles on an element, use setStyles instead. 
-                It takes a dictionary of attribute, value pairs and applies it all in one go (faster)
-
-                To remove a style, set its value to empty string.
-                When all styles are removed, the "style" attribute will be nullified.
-
-            @param styleName - The name of the style element
-
-            @param styleValue - The value of which to assign the style element
-
-              NOTE: dash-names (like padding-top) are expected here
-
-            @return - String of current value of "style" after change is made.
-        '''
-        return self.setStyles( {styleName : styleValue} )
-
-    def setStyles(self, styleUpdatesDict):
-        '''
-            setStyles - Sets one or more style params. 
-                This all happens in one shot, so it is much much faster than calling setStyle for every value.
-
-                To remove a style, set its value to empty string.
-                When all styles are removed, the "style" attribute will be nullified.
-
-            @param styleUpdatesDict - Dictionary of attribute : value styles.
-
-              NOTE: dash-names (like padding-top) are expected here.
-
-            @return - String of current value of "style" after change is made.
-        '''
-        styleDict = self.getStyleDict()
-
-        for newName, newValue in styleUpdatesDict.items():
-            if newValue:
-                # If replacing, just set/override it
-                styleDict[newName] = newValue
-            elif newName in styleDict:
-                # Delete if present and empty value passed
-                del styleDict[newName]
-
-        if styleDict:
-            # If anything left, build a str
-            styleStr = '; '.join([name + ': ' + value for name, value in styleDict.items()])
-            self.setAttribute('style', styleStr)
-            return styleStr
-        else:
-            # Last item removed, so remove attribute.
-            self.removeAttribute('style')
-            return ''
+#    def getStyleDict(self):
+#        '''
+#            getStyleDict - Returns a dictionary of style attribute name:value. 
+#
+#            @return dict <str:str> style attribute names to values. Impl is OrderedDict and will match positioning in string value
+#        '''
+#        return self._styleDict
+#
+#    def getStyle(self, styleName):
+#        '''
+#            getStyle - Gets the value of a style paramater, part of the "style" attribute
+#
+#            @param styleName - The name of the style
+#
+#              NOTE: dash-names (like padding-top) are expected here
+#
+#            @return - String of the value of the style. '' is no value.
+#        '''
+#        return self.getStyleDict().get(styleName.lower()) or ''
+#        
+#
+#    def setStyle(self, styleName, styleValue):
+#        '''
+#            setStyle - Sets a style param. Example: "display", "block"
+#
+#                If you need to set many styles on an element, use setStyles instead. 
+#                It takes a dictionary of attribute, value pairs and applies it all in one go (faster)
+#
+#                To remove a style, set its value to empty string.
+#                When all styles are removed, the "style" attribute will be nullified.
+#
+#            @param styleName - The name of the style element
+#
+#            @param styleValue - The value of which to assign the style element
+#
+#              NOTE: dash-names (like padding-top) are expected here
+#
+#            @return - String of current value of "style" after change is made.
+#        '''
+#        return self.setStyles( {styleName : styleValue} )
+#
+#    def setStyles(self, styleUpdatesDict):
+#        '''
+#            setStyles - Sets one or more style params. 
+#                This all happens in one shot, so it is much much faster than calling setStyle for every value.
+#
+#                To remove a style, set its value to empty string.
+#                When all styles are removed, the "style" attribute will be nullified.
+#
+#            @param styleUpdatesDict - Dictionary of attribute : value styles.
+#
+#              NOTE: dash-names (like padding-top) are expected here.
+#
+#            @return - String of current value of "style" after change is made.
+#        '''
+#        styleDict = self.getStyleDict()
+#
+#        for newName, newValue in styleUpdatesDict.items():
+#            if newValue:
+#                # If replacing, just set/override it
+#                styleDict[newName] = newValue
+#            elif newName in styleDict:
+#                # Delete if present and empty value passed
+#                del styleDict[newName]
+#
+#        if styleDict:
+#            # If anything left, build a str
+#            styleStr = '; '.join([name + ': ' + value for name, value in styleDict.items()])
+#            self.setAttribute('style', styleStr)
+#            return styleStr
+#        else:
+#            # Last item removed, so remove attribute.
+#            self.removeAttribute('style')
+#            return ''
 
     def _asStr(self):
+        '''
+            _asStr - Get the string representation of this style
+
+              @return <str> - A string representation of this style (semicolon separated, key: value format)
+        '''
+
         styleDict = self._styleDict
         if styleDict:
             return '; '.join([name + ': ' + value for name, value in styleDict.items()])
@@ -437,8 +635,28 @@ class StyleAttribute(object):
 
 
     def __eq__(self, other):
+        '''
+            __eq__ - Test if two "style" tag properties are equal.
+
+                NOTE: This differs from javascript. In javascript, no two styles equal eachother, it's
+                        an identity comparison not a value comparison.
+
+                      I don't understand how that is useful, but in a future version we may choose to adopt
+                        that "feature" and export comparison into a different "isSaneAs(otherStyle)" function
+                
+                @param other<StyleAttribute> - The other style attribute map.
+        '''
+
         selfDict = self._styleDict
-        otherDict = other._styleDict
+
+        # Check if "other" is a string and convert it to a StyleAttribute if so, for comparison
+        #   (this also allows any-order, i.e. 
+        #     StyleAttribute("float: left; font-weight: bold") == StyleAttribute("font-weight: bold; float: left")
+        try:
+            otherDict = other._styleDict
+        except:
+            otherStyle = StyleAttribute(other)
+            otherDict = otherStyle._styleDict
 
         selfDictKeys = list(selfDict.keys())
         otherDictKeys = list(otherDict.keys())
